@@ -7,7 +7,6 @@ from typing import Any, Dict, Iterable, List, Optional
 import psycopg2
 import psycopg2.extras
 
-from relay.blockchain import currency_network_events
 from relay.blockchain.events import BlockchainEvent, TLNetworkEvent
 
 # proxy.get_all_events just asks for these network events. so we need the list
@@ -21,6 +20,16 @@ def connect(dsn):
     return psycopg2.connect(dsn, cursor_factory=psycopg2.extras.RealDictCursor)
 
 
+def get_latest_ethindex_block_number(conn):
+    with conn.cursor() as cur:
+        cur.execute("""select * from sync where syncid='default'""")
+        row = cur.fetchone()
+        if row:
+            return row["last_block_number"]
+        else:
+            raise RuntimeError("Could not determine current block number")
+
+
 # EventsQuery is used to store a where block together with required parameters
 # EthindexDB._run_events_query uses this to build and run a complete query.
 EventsQuery = collections.namedtuple("EventsQuery", ["where_block", "params"])
@@ -31,15 +40,17 @@ class EventBuilder:
     pretty much the same logic like relay.blockchain.Proxy (or its
     subclasses). The handling for timestamps is different. We also don't ask
     web3 for the currentBlock. It's passed in from the caller.
-    contract_types is a mapping from contract_address to type of the contract
+    address_to_contract_types is a mapping from contract_address to type of the contract
     to resolve naming collisions of events when used with different contract types.
     """
 
     def __init__(
-        self, _event_builders: Dict[str, Any], contract_types: Dict[str, str] = None
+        self,
+        _event_builders: Dict[str, Any],
+        address_to_contract_types: Dict[str, str] = None,
     ) -> None:
         self.event_builders = _event_builders
-        self.contract_types = contract_types
+        self.address_to_contract_types = address_to_contract_types
 
     def build_events(
         self, events: List[Any], current_blocknumber: int
@@ -53,9 +64,9 @@ class EventBuilder:
     def _build_event(self, event: Any, current_blocknumber: int) -> BlockchainEvent:
         event_type: str = event.get("event")
         timestamp: int = event.get("timestamp")
-        if self.contract_types is not None:
+        if self.address_to_contract_types is not None:
             address: str = event.get("address")
-            contract_type = self.contract_types[address]
+            contract_type = self.address_to_contract_types[address]
         else:
             contract_type = ""
         return self.event_builders[contract_type + event_type](
@@ -84,7 +95,7 @@ order_by_default_sort_order = """ ORDER BY blocknumber, transactionIndex, logInd
 
 class EthindexDB:
     """EthIndexDB provides an interface for ethindex database
-    it used to access events from the database.
+    it is used to access events from the database.
     """
 
     def __init__(
@@ -94,14 +105,16 @@ class EthindexDB:
         event_builders,
         from_to_types,
         address=None,
-        contract_types=None,
+        address_to_contract_types: Dict[str, str] = None,
     ):
         self.conn = conn
         self.default_address = address
         self.standard_event_types = standard_event_types
-        self.event_builder = EventBuilder(event_builders, contract_types=contract_types)
+        self.event_builder = EventBuilder(
+            event_builders, address_to_contract_types=address_to_contract_types
+        )
         self.from_to_types = from_to_types
-        self.contract_types = contract_types
+        self.address_to_contract_types = address_to_contract_types
 
     @property
     def event_types(self):
@@ -111,13 +124,7 @@ class EthindexDB:
         return self.event_builder.build_events(rows, self._get_current_blocknumber())
 
     def _get_current_blocknumber(self):
-        with self.conn.cursor() as cur:
-            cur.execute("""select * from sync where syncid='default'""")
-            row = cur.fetchone()
-            if row:
-                return row["last_block_number"]
-            else:
-                raise RuntimeError("Could not determine current block number")
+        return get_latest_ethindex_block_number(self.conn)
 
     def _get_addr(self, address):
         """all the methods here take an address argument
@@ -126,7 +133,7 @@ class EthindexDB:
         default_address.
         """
         r = address or self.default_address
-        assert r, "no network address passed in and no default network address given"
+        assert r, "no contract address passed in and no default contract address given"
         return r
 
     def _get_standard_event_types(
@@ -199,7 +206,7 @@ class EthindexDB:
         from_block: int = 0,
         contract_address: str = None,
     ) -> List[BlockchainEvent]:
-        # This function only works properly for many contracts if self.contract_types is properly set
+        # This function only works properly for many contracts if self.address_to_contract_types is properly set
         # TODO Refactor and move somewhere else
         contract_address = contract_address or self.default_address
 
@@ -216,7 +223,8 @@ class EthindexDB:
             args.append(contract_address)
         else:
             query_string += "AND address in %s "
-            args.append(tuple(self.contract_types))
+            # We assume self.address_to_contract_types is properly set and not None
+            args.append(tuple(self.address_to_contract_types))  # type: ignore
 
         query = EventsQuery(query_string, args)
 
@@ -242,7 +250,7 @@ class EthindexDB:
         return events
 
     def get_events(
-        self, event_type, from_block=0, contract_address: str = None,
+        self, event_type, from_block: int = 0, contract_address: str = None
     ) -> List[BlockchainEvent]:
         contract_address = self._get_addr(contract_address)
         query = EventsQuery(
@@ -263,6 +271,116 @@ class EthindexDB:
 
         return events
 
+    def get_events_from_to(
+        self,
+        event_types: Iterable[str] = None,
+        start_time: int = 0,
+        end_time: int = None,
+        contract_address: str = None,
+        from_address: str = None,
+        to_address: str = None,
+    ):
+        """
+        Get events with given parameters.
+
+        If `from_address` and `to_address` are set, will find all events with matching `from` and `to` for given event
+        types. e.g. for a TrustlineUpdate it will match for `_creditor` and `_debtor`.
+
+        If no event_types is given, uses self.standard_event_types.
+        """
+        if event_types is None:
+            event_types = self.standard_event_types
+
+        query_strings = []
+        query_params: List[Any] = []
+
+        if from_address is not None or to_address is not None:
+            from_to_string, from_to_args = self.get_query_for_from_to(
+                event_types, from_address, to_address
+            )
+            query_strings.append(from_to_string)
+            query_params += from_to_args
+        elif event_types is not None:
+            query_strings.append("eventName in %s")
+            query_params.append(tuple(event_types))
+
+        if start_time != 0:
+            query_strings.append("timestamp>=%s")
+            query_params.append(start_time)
+        if end_time is not None:
+            query_strings.append("timestamp<=%s")
+            query_params.append(end_time)
+
+        contract_address = self._get_addr(contract_address)
+        query_strings.append("address=%s")
+        query_params.append(contract_address)
+
+        query = EventsQuery(" AND ".join(query_strings), query_params)
+
+        events = self._run_events_query(query)
+
+        logger.debug(
+            "get_events_from_to(%s, %s, %s, %s, %s, %s) -> %s rows",
+            event_types,
+            start_time,
+            end_time,
+            contract_address,
+            from_address,
+            to_address,
+            len(events),
+        )
+
+        return events
+
+    def get_query_for_from_to(self, event_types, from_address, to_address):
+        """
+        Make a query string for finding events of types `event_types` with matching `from_address` and `to_address`
+
+        Assumes self.from_to_types is properly set for given event_types
+        """
+
+        if from_address is None and to_address is None:
+            raise ValueError(
+                "Cannot filter for from_address and to_address if addresses are None"
+            )
+
+        event_type_from_to_query_strings = []
+        query_params = []
+
+        for event_type in event_types:
+            if event_type not in self.from_to_types.keys():
+                raise ValueError(
+                    f"No `from_to_types` for given event type: {event_type}"
+                )
+            query_params.append(event_type)
+            user_types = self.from_to_types[event_type]
+
+            from_string = ""
+            to_string = ""
+            if from_address is not None:
+                from_string = f"args->>'{user_types[0]}'=%s"
+                query_params.append(from_address)
+            if to_address is not None:
+                to_string = f"args->>'{user_types[1]}'=%s"
+                query_params.append(to_address)
+
+            coordinator = ""
+            if from_string != "" and to_string != "":
+                coordinator = " AND "
+            from_to_query_string = from_string + coordinator + to_string
+
+            event_type_from_to_string = (
+                "(eventName=%s AND (" + from_to_query_string + "))"
+            )
+
+            event_type_from_to_query_strings.append(event_type_from_to_string)
+
+        final_string = " OR ".join(
+            string for string in event_type_from_to_query_strings
+        )
+
+        return final_string, query_params
+
     def get_all_events(
         self,
         from_block: int = 0,
@@ -322,7 +440,7 @@ class EthindexDB:
         event_types = self._get_standard_event_types(event_types)
 
         query = EventsQuery(
-            f"""blockHash=%s
+            """blockHash=%s
             AND eventName in %s
             AND transactionHash IN
                 (SELECT transactionHash "transactionHash" FROM events WHERE blockHash=%s AND logIndex=%s LIMIT 1)
@@ -365,7 +483,10 @@ class CurrencyNetworkEthindexDB(EthindexDB):
         return self.get_user_events(event_type, user_address, from_block)
 
     def get_all_network_events(
-        self, user_address: str = None, from_block: int = 0
+        self,
+        user_address: str = None,
+        from_block: int = 0,
+        event_types: Iterable[str] = None,
     ) -> List[BlockchainEvent]:
         if self.default_address is None:
             # if the default address is not set we will get events from non currency network contracts
@@ -373,7 +494,7 @@ class CurrencyNetworkEthindexDB(EthindexDB):
                 "Cannot get all network events if CurrencyNetworkEthindexDB address is not set."
             )
         return self.get_all_contract_events(
-            currency_network_events.standard_event_types, user_address, from_block,
+            event_types=event_types, user_address=user_address, from_block=from_block,
         )
 
     def get_trustline_events(
@@ -440,7 +561,7 @@ class ExchangeEthindexDB(EthindexDB):
 
         event_types = self._get_standard_event_types([type])
 
-        query_string = f"""blockNumber>=%s
+        query_string = """blockNumber>=%s
                             AND eventName in %s
                             AND address in %S"""
         args = [from_block, event_types, all_exchange_addresses]
